/*
   log.c - logging funtions

   Copyright (C) 2002 Arthur de Jong.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/


#include "config.h"

#include <stdio.h>
#include <sys/types.h>
#include <unistd.h>
#include <syslog.h>
#include <stdarg.h>

#include "log.h"


/* LOG_DEBUG LOG_INFO [LOG_NOTICE] [LOG_WARNING] LOG_ERR LOG_CRIT */
#ifdef DEBUG
int cvsd_loglevel=LOG_DEBUG;
#else /* DEBUG */
int cvsd_loglevel=LOG_INFO;
#endif /* not DEBUG */


/* LOG_SYSLOG LOG_STDERR */
static int cvsd_logmode;


/* the file that is logged to */
static FILE *cvsd_logfile;


/* initialize the logging system */
void initlog(int mode)
{
  cvsd_logmode=mode;
  if (cvsd_logmode&CVSD_LOG_SYSLOG)
  {
    openlog(PACKAGE,LOG_PID,LOG_DAEMON);
  }
  if (cvsd_logmode&CVSD_LOG_STDERR)
  {
    /* duplicate stderr so it is still available later */
    cvsd_logfile=fdopen(dup(2),"a");
  }
}


/* log the given message using the configured logging method */
void log(int pri,const char *format, ...)
{
  int res;
  /* TODO: make this something better */
  #define maxbufferlen 80
  char buffer[maxbufferlen];
  va_list ap;
  /* make the message */
  va_start(ap,format);
  res=vsnprintf(buffer,maxbufferlen,format,ap);
  if ((res<0)||(res>=maxbufferlen))
  {
    /* truncate with "..." */
    buffer[maxbufferlen-2]='.';
    buffer[maxbufferlen-3]='.';
    buffer[maxbufferlen-4]='.';
  }
  buffer[maxbufferlen-1]='\0';
  va_end(ap);
  /* do the logging */
  if ((pri<=cvsd_loglevel)&&(cvsd_logmode&CVSD_LOG_SYSLOG))
    syslog(LOG_DEBUG,"%s",buffer);
  if (cvsd_logmode&CVSD_LOG_STDERR)
    fprintf(cvsd_logfile,"%s[%d]: %s\n",PACKAGE,getpid(),buffer);
}
